/* FTUI Plugin
 * Copyright (c) 2015-2018 Mario Stephan <mstephan@shared-files.de>
 * Under MIT License (http://www.opensource.org/licenses/mit-license.php)
 */

/* global ftui:true, Modul_famultibutton:true */

"use strict";

var Modul_playsound = function () {

    function startStream(elem) {

        //stop all streams
        me.elements.each(function (index, el) {
            stopStream($(this));
        });

        var audio = elem.data('audio');
        var volume = elem.data('volume');

        audio.src = elem.data('src');
        audio.type = elem.data('type');
        audio.load();
        audio.play().catch(function (error) {
            ftui.log(1, "Error: " + error);
        });
        if ($.isNumeric(volume)) {
            audio.volume = parseInt(volume) / 100.0;
        }
    }

    function stopStream(elem) {

        var audio = elem.data('audio');
        audio.pause();
        audio.src = null;
        audio.load();
    }

    function init() {

        me.elements = $('div[data-type="' + me.widgetname + '"]:not([data-ready])', me.area);
        me.elements.each(function (index) {
            var elem = $(this);
            elem.attr("data-ready", "");

            elem.initData('volume', 'volume');
            elem.initData('audio', new Audio());
            elem.initData('type', 'audio/mpeg');

            if (!elem.isUrlData('url')) {
                me.addReading(elem, 'url');
            } else {
                elem.data('src',elem.data('url'));
            }

            if (!$.isNumeric(elem.data('volume'))) {
                me.addReading(elem, 'volume');
            }
            me.init_attr(elem);
            me.init_ui(elem);
			elem.hide();
        });
    }

    function update(dev, par) {

        // update from normal state reading
        me.elements.filterDeviceReading('get', dev, par)
            .each(function (index) {
                var elem = $(this);
                var value = elem.getReading('get').val;

                if (elem.matchingState('get', value) === 'on') {
					startStream(elem);
                }
                if (elem.matchingState('get', value) === 'off') {
					stopStream(elem);
                }
            });

        // update from extra reading for volume
        me.elements.filterDeviceReading('volume', dev, par)
            .each(function (idx) {
                var elem = $(this);
                var volume = elem.getReading('volume').val;
                if ($.isNumeric(volume)) {
                    ftui.log(3, ['playstream - set volume to :', parseInt(volume) / 100.0].join(''));
                    elem.data('audio').volume = parseInt(volume) / 100.0;
                }
            });

        // update from extra reading for url
        me.elements.filterDeviceReading('url', dev, par)
            .each(function (idx) {
                var elem = $(this);
                var url = elem.getReading('url').val;
                if (ftui.isValid(url)) {
                    ftui.log(3, ['playstream - set url to :', url].join(''));
                    elem.data('src', url);
                    if ( !elem.data('audio').paused) {
                        startStream(elem);
                    }
                }
            });
    }

    // public
    // inherit members from base class
    var me = $.extend(new Modul_widget(), {
        //override members
        widgetname: 'playsound',
        init: init,
        update: update,
    });

    return me;
};
